<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Place;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use App\Helpers\SMSClient;

class AuthController extends Controller
{
    /**
     * Constructs a new instance of the class.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth:api', [
            'except' => [
                'login',
                'register',
                'webLogin',
                'webIndex',
                'webLogout',
                'loginWithCode',
                'sendVerificationCode',
                'webForgetPassword' ,
                'webLoginWithCode',
                'notAuthorized',
                ]
            ]);
    }

    /**
     * A function that handles unauthorized access.
     *
     * @throws Some_Exception_Class not authorized exception
     * @return Some_Return_Value the error response
     */
    public function notAuthorized(){
        return response()->error(403, "not authorized!", []);
    }

    /**
     * Logs in a user using the provided request data.
     *
     * @param Request $request the HTTP request object containing the username and password
     * @throws Some_Exception_Class if the validation fails or the user credentials are invalid
     * @return Response the HTTP response containing the login data and authentication token
     */
    public function login(Request $request)
    {
        $request->validate([
            'username' => 'required|string',
            'password' => 'required|string',
        ]);
        $credentials = $request->only('username', 'password');
        $token = Auth::guard('api')->attempt($credentials);
        if (!$token) {
            return response()->unauthorized();
        }

        try{
            $user = Auth::guard('api')->user();
            if($user->default_place != null){
                $place = Place::find($user->default_place);
            }else{
                $place = $user->places->first();
            }
            if($user->superadmin)
                $place = Place::first();
            if($place == null)
                return response()->error(403,"No place assigned!",[]);
            $token = Auth::guard('api')->claims(['default_place'=> $place->id])->attempt($credentials);
            if($place != null){
                // $token = Auth::guard('api')->claims(['default_place_id'=> $place->id])->attempt();
                $placeLogo = $place->logo;
                $placeUrl = $place->url;
            }
            return response()->standard("Login successfully", [
                'user' => $user,
                "place_id" => $place->id,
                "place_name" => $place->name,
                "place_logo" => $placeLogo,
                "place_url" => $placeUrl,
                'authorization' => [
                    'token' => $token,
                    'type' => 'bearer',
                ],
            ]);
        }catch(Exception $e){
            return response()->unauthorized();
        }
    }

    /**
     * Registers a new user.
     *
     * @param Request $request The HTTP request object.
     * @throws Some_Exception_Class If the validation fails.
     * @return Some_Return_Value The response message and user UUID.
     */
    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'username' => 'required|string|max:255',
            'password' => 'required|string|min:6',
            'place_uuid' => 'required|string|size:36|exists:places,id'
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }

        $user = User::create([
            'username' => $request->username,
            'password' => Hash::make($request->password),
            'place_uuid' => $request->place_uuid
        ]);

        return response()->standard('User created successfully', [
            'user_uuid' => $user->id
        ]);
    }

    /**
     * Logout the user from the API.
     *
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function logout()
    {
        Auth::guard('api')->logout();
        return response()->standard('Successfully logged out', []);
    }

    /**
     * Validates the request data and sends a verification code to the user.
     *
     * @param Request $request The HTTP request object containing the data to be validated.
     *                        The request must have the following fields: country_code (string)
     *                        and phone (string).
     * @throws Some_Exception_Class If the request data fails validation.
     * @return Some_Return_Value The response containing the verification code and the user's username
     *                           if the user is found.
     */
    public function sendVerificationCode(Request $request){
        $validator = Validator::make($request->all(), [
            'country_code' => ['required', 'exists:users,country_code'],
            'phone' => ['required', 'exists:users,phone']
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        $data = $validator->validated();
        $user = User::where('country_code', $data['country_code'])->where('phone', $data['phone'])->get();
        if(count($user)==1){
            $user = $user[0];
            $user->verification_code =  Str::random(5);
            $user->code_exp = date('Y-m-d H:i:s', time()+240);
            $user->save();
            $smsResult = SMSClient::send($user->country_code . $user->phone, $user->verification_code." is your verification code");
            return response()->standard("", ["username" => $user->username]);
        }

        else{
            return response()->error("user not found", []);
        }

    }

    /**
     * Logs in the user using a verification code.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class If there is an exception.
     * @return Some_Return_Value The response value.
     */
    public function loginWithCode(Request $request){
        $validator = Validator::make($request->all(), [
            "username" => ['required', 'exists:users,username'],
            "verification_code" => ['required', 'exists:users,verification_code']
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        try{
            $data = $validator->validated();
            $user = User::where('username', $data['username'])->where('verification_code', $data['verification_code'])->where('code_exp','>=',date('Y-m-d H:i:s',strtotime('now')))->get();
            if(count($user)==1){
                $user = $user[0];
                return response()->standard("", ["token"=>Auth::guard('api')->fromUser($user)]);
            }else{
                return response()->error(1001, "user not found", []);
            }
        }catch(Exception $e){
            return response()->error(1001, $e->getMessage());
        }


    }

    /**
     * Reset the user's password.
     *
     * @param Request $request The HTTP request object.
     * @throws Some_Exception_Class If the validation fails.
     * @return Some_Return_Value The response indicating success or failure.
     */
    public function resetPassword(Request $request){
        $validator = Validator::make($request->all(), [
            'password' => 'required|string|min:6'
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        $user = Auth::user();
        $user->password =  Hash::make($request->password);
        $user->save();
        return response()->standard("Password reset successfully", []);
    }

    /**
     * Refreshes the API token for the authenticated user.
     *
     * @return \Illuminate\Http\Response
     */
    public function refresh()
    {
        return response()->standard ("",[
            'user' => Auth::user(),
            'authorisation' => [
                'token' => Auth::refresh(),
                'type' => 'bearer',
            ]
        ]);
    }


    /**
     * A description of the entire PHP function.
     *
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function webIndex(){
        return view("login");
    }

    /**
     * Logs in a user through the web interface.
     *
     * @param Request $request The HTTP request object.
     * @throws ValidationError If the provided credentials are invalid.
     * @return RedirectResponse|View Returns a redirect response or a view.
     */
    public function webLogin(Request $request){
        $credentials = $request->validate([
            'username' => ['required'],
            'password' => ['required'],
        ]);
        if (Auth::guard('web')->attempt($credentials)) {
            $request->session()->regenerate();
            $user = auth()->guard('web')->user();
            if($user->is_superadmin){
                return redirect()->intended('/nova');//web/adminPannel');
            }elseif ($user->is_manager) {
                return redirect()->intended('/nova');//web/manager/home');
            }elseif ($user->is_reception){
                return view('reception_pannel');
            }
        }
        return back()->withErrors([
            'error' => 'The provided credentials do not match our records.',
        ])->onlyInput('error');
    }

    /**
     * Logs out the user from the web session.
     *
     * @param Request $request The HTTP request object.
     * @return RedirectResponse The response redirecting to the '/web' page.
     */
    public function webLogout(Request $request){
        Auth::guard('web')->logout();

        $request->session()->invalidate();

        $request->session()->regenerateToken();

        return redirect('/web');
    }

    /**
     * A function to render the forget password view.
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function webForgetPassword(){
        return view('forgetPassword', ["error" => session('error')]);
    }

    /**
     * Authenticate a user using a verification code in a web login.
     *
     * @param Request $request The HTTP request object.
     * @throws Some_Exception_Class If validation fails.
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Contracts\View\Factory
     */
    public function webLoginWithCode(Request $request){
        $validator = Validator::make($request->all(), [
            "username" => ['required', 'exists:users,username'],
            "verification_code" => ['required', 'exists:users,verification_code']
        ]);
        if($validator->fails()){
            return redirect()->back()->with(['error'=>$validator->errors()]);
        }
        $data = $validator->validated();
        $user = User::where('username', $data['username'])->where('verification_code', $data['verification_code'])->where('code_exp','>',date('Y-m-d H:i:s',strtotime('now')))->get();
        if(count($user)==1){
            $user = $user[0];
            Auth::guard('web')->loginUsingId($user->id, false);
            // Auth::guard('web')->updateSession($user->id, false);
            // $request->session()->regenerate();
            $user = auth()->guard('web')->user();
            if($user->is_superadmin){
                return redirect()->intended('/web/showManager/'.$user->id);
            }elseif ($user->is_manager) {
                return redirect()->intended('web/manager/staff/'.$user->id);
            }elseif ($user->is_reception){
                return view('reception_pannel');
            }
        }else{
            return redirect()->back()->with(['error'=>"Verification code is invalid!"]);
        }

    }
}
