<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Notifications\Notifiable;
use App\Enums\CarStatus;
use App\Helpers\TimeModification;
use DateTime;
use DateTimeZone;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Support\Facades\DB;
use Psy\TabCompletion\Matcher\FunctionsMatcher;

class Assignment extends Model
{
    use SoftDeletes, HasUuids, Notifiable;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'firstname',
        'lastname',
        'country_code',
        'phone',
        'email',
        'is_guest',
        'room',
        'checkout_date',
        'car_plate',
        'car_make',
        'car_color',
        'carwash',
        'satus',
        'charge',
        'order_ref',
        'user_uuid',
        'ticket_uuid',
        'assigned_to',
        'assigned_at',
        'requested_at',
        'paid',
        'extra_service',
        'send_sms',
        'sms_id',
        'sms_delivered',
        'discount',
        "payment_ref",
        "payment_status",
        "cash_reciever",
        "applied_promotion",
        'level',
        'slot',
        'hanger',
    ];
    protected $appends = ['time_to_now', 'place', 'status_color', 'hasDamage'];
    protected $casts = [
        'status' => CarStatus::class,
        'checkout_date' => 'date',
    ];

    protected function assignedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->ticket->place->time_zone)
        );
    }

    protected function createdAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->ticket->place->time_zone)
        );
    }

    protected function updatedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->ticket->place->time_zone)
        );
    }

    protected function deletedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->ticket->place->time_zone)
        );
    }

    protected function requestedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->ticket->place->time_zone)
        );
    }

    /**
     * Calculate the time duration between the current time and the updated_at timestamp of the object.
     *
     * @return array An array containing the number of days, hours, and minutes.
     */
    public function getTimeToNowAttribute()
    {
        $diff = strtotime(TimeModification::setNowTimeZone($this->ticket->place->time_zone)) - strtotime($this->updated_at->format('Y-m-d H:i:s'));
        $day = floor($diff/24/3600);
        $diff -= 24*3600*$day;
        $hour = floor($diff/3600);
        $diff -= 3600*$hour;
        $min = floor($diff/60);
        return [
            "day" => $day,
            "hour" => $hour,
            "min" => $min
        ];
    }
    public function getHasDamageAttribute()
    {
        $damage = CarDamage::where('assignment_uuid', $this->id)->first();
         if ($damage == null) {
             return 0;
        } else {
             return 1;
        }
    }
    public function getPriceAttribute(){
        return
            \App\Http\Controllers\CustomerController::bill($this);
    }

    public function getPaymentsAttribute(){
        return \App\Http\Controllers\CustomerController::bill($this)['payments'];
    }
    /**
     * Retrieves the status color attribute based on the current status of the car.
     *
     * @return string The color code representing the status color.
     */
    public function getStatusColorAttribute()
    {
        if($this->status == CarStatus::REQUESTED_PAID ||
            $this->status == CarStatus::REQUESTED_NOT_PAID){
                if($this->assigned_to == null)
                    return "#1971e3"; // blue
                else
                    return "#ffe319"; // yellow
            }
        if($this->status == CarStatus::READY_PAID)
            return "#13a10d"; //green
        if($this->status == CarStatus::READY_NOT_PAID)
            return "#e31919"; //red
        if($this->status == CarStatus::PARKING)
            return "#dcdcdc"; // grey
        if($this->status == CarStatus::OUTDOOR)
            return "#a10a76"; // pillow
    }

    /**
     * Retrieves the place attribute of the current object.
     *
     * @return string The name of the place.
     */
    public function getPlaceAttribute()
    {
        return $this->ticket->place->name;
    }

    /**
     * Get the user associated with the model.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo The BelongsTo relationship.
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class, 'user_uuid')->withTrashed();
    }

    /**
     * Get the user assigned to this model.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function assignedTo(): BelongsTo
    {
        // Return the relationship where the assigned_to column matches the user's ID.
        return $this->belongsTo(User::class, 'assigned_to')->withTrashed();
    }

    public function cashReciever(): BelongsTo
    {
        // Return the relationship where the cash_reciever column matches the user's ID.
        return $this->belongsTo(User::class, 'cash_reciever')->withTrashed();
    }
    /**
     * Get the ticket associated with this model.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function ticket(): BelongsTo
    {
        return $this->belongsTo(Ticket::class, 'ticket_uuid')->withTrashed();
    }

    /**
     * Retrieves the car damages associated with this assignment.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function carDamages(): HasMany
    {
        return $this->hasMany(CarDamage::class, 'assignment_uuid');
    }
}
