<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use App\Models\User;
use App\Models\Role;
use App\Models\Promotion;
use Illuminate\Database\Eloquent\Casts\Attribute;
use App\Enums\PlaceType;

use App\Enums\RoleType;
use App\Helpers\TimeModification;
use Illuminate\Database\Eloquent\Casts\AsEnumCollection;

class Place extends Model
{
    use HasUuids, SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'type',
        'description',
        'address',
        'logo',
        'url',
        'phone',
        'country_code',
        'time_zone',
        'all_week_price',
        'week_days_price',
        'friday_price',
        'weekend_price',
        'carwash_price',
        'fuel_price',
        'tax',
        'capacity',
        'txt_msg',
    ];

    protected $appends = ['parking_availability'];

    /**
     * Write code on Method
     *
     * @return response()
     */
    protected $casts = [
        'type' => PlaceType::class,
    ];

    protected function createdAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->time_zone)
        );
    }

    protected function updatedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->time_zone)
        );
    }

    protected function deletedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->time_zone)
        );
    }


    /**
     * Get the users that belong to the role.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function users(): BelongsToMany
    {
        return $this->belongsToMany(
            User::class,                       // The related model
            'roles',                           // The pivot table
            'place_uuid',                      // The foreign key of this model
            'user_uuid'                        // The foreign key of the related model
        )->using(Role::class)                  // The intermediate model
        ->withPivot('permissions')             // The pivot columns to retrieve
        ->withTimestamps()                     // The timestamp columns
        ->whereNull('roles.deleted_at');
    }
    /**
     * Get the tickets associated with the place.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function tickets(): HasMany
    {
        // Return the relationship between the Place and Ticket models
        return $this->hasMany(Ticket::class, 'place_uuid');
    }
    /**
     * Get the promotions associated with the place.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function promotions(): HasMany
    {
        // Return the has many relationship between the Place model and the Promotion model
        return $this->hasMany(Promotion::class, 'place_uuid');
    }

    /**
     * Get the ads that belong to the place.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function advertisements(): BelongsToMany
    {
        return $this->belongsToMany(
            Advertisement::class,                       // The related model
            'place_ads',                           // The pivot table
            'place_uuid',                      // The foreign key of this model
            'ad_uuid'                        // The foreign key of the related model
        )->using(PlaceAds::class)               // The intermediate model
        ->withPivot('is_active')              // The pivot columns to retrieve
        ->withTimestamps();                    // The timestamp columns
    }

    /**
     * Calculates the availability of parking spaces.
     *
     * @return int The number of available parking spaces.
     */
    public function getParkingAvailabilityAttribute(){
        $availability = $this->capacity - $this->tickets()->where('status', 1)->count();
        if($availability < 0){
            return 0;
        }
        return $availability;
    }

}
