<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use PHPOpenSourceSaver\JWTAuth\Contracts\JWTSubject;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use App\Models\Place;
use App\Models\Role;
use Illuminate\Database\Eloquent\Casts\AsCollection;
use App\Enums\RoleType;
use App\Helpers\TimeModification;
use Illuminate\Database\Eloquent\Casts\AsEnumCollection;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Rappasoft\LaravelAuthenticationLog\Traits\AuthenticationLoggable;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

class User extends Authenticatable implements JWTSubject
{
    use HasApiTokens, SoftDeletes, Notifiable, HasUuids, AuthenticationLoggable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'username',
        'email',
        'password',
        'firstname',
        'lastname',
        'country_code',
        'phone',
        'verification_code',
        'code_exp',
        'email_verified_at',
        'activated_at',
        'superadmin',
        'avatar',
        'national_id',
        'national_id_exp',
        'last_login_at',
        'last_logout_at',
        'ip',
        'default_place',
    ];


    protected $appends = ['mix_phone'];

    public function getMixPhoneAttribute()
    {
        $number = $this->country_code .' '. $this->phone;
        return $number;
    }

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'national_id_exp' => 'date',
        'password' => 'hashed',
    ];


    protected function createdAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::userSetTimeZone($this, $time)
        );
    }

    protected function updatedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::userSetTimeZone($this, $time)
        );
    }

    protected function deletedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::userSetTimeZone($this, $time)
        );
    }


    /**
     * Get the places that the user belongs to.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function places(): BelongsToMany
    {
        // Define the relationship between the User model and the Place model
        // using the "roles" table as the intermediate table.
        // The "user_uuid" column in the "roles" table is used as the foreign key
        // for the User model, and the "place_uuid" column is used as the foreign key
        // for the Place model.
        // The "Role" model is used to customize the intermediate table.
        // The "permissions" column and the timestamp columns are included in the pivot table.
        return $this->belongsToMany(Place::class, 'roles', 'user_uuid', 'place_uuid')
            ->using(Role::class)
            ->withPivot('permissions')
            ->withTimestamps();
    }

    /**
     * Get the identifier that will be stored in the subject claim of the JWT.
     *
     * @return mixed The identifier of the JWT.
     */
    public function getJWTIdentifier()
    {
        // Return the key of the JWT.
        return $this->getKey();
    }

    /**
     * Returns an empty key value array.
     *
     * This function is used to provide any custom claims that need to be added to the JWT (JSON Web Token).
     * Custom claims are additional information that can be included in the JWT payload.
     *
     * @return array An empty key value array.
     */
    public function getJWTCustomClaims()
    {
        return [];
    }
    /**
     * Get the assignment that owns the CarDamage.
     */
    public function defaultPlace(): BelongsTo
    {
        return $this->belongsTo(Place::class, 'default_place')->withTrashed();
    }

}
